@@ -65,6 +65,23 @@ sub is_support_model_ext {
     $self->model_ext($ret || 0);
     return $ret;
 }
+sub get_model_status{
+    my $self = shift;
+    if(     defined $self->model_status->{user} 
+        and defined $self->model_status->{friend} 
+        and defined $self->model_status->{group}
+    ){
+        my $is_fail =
+               $self->model_status->{user} == 0
+            && $self->model_status->{friend} == 0
+            && $self->model_status->{group} == 0
+        ;
+        return $is_fail?0:1;
+    }
+    else{
+        return -1;
+    }
+}
 sub get_csrf_token {
     my $self = shift;
     if(not $self->is_support_model_ext){
@@ -135,12 +152,25 @@ sub update_user {
     my $user_info = $self->_get_user_info();
     unless ( defined $user_info ) {
         $self->warn("更新个人信息失败\n");
-        $self->emit("model_update_fail");
+        $self->emit("model_update"=>"user",0);
         return;
     }       
     $self->user($self->new_user($user_info));
+    $self->emit("model_update"=>"user",1);
 }
 
+sub remove_friend {
+    my $self = shift;
+    my $friend = shift;
+    $self->die("不支持的数据类型\n") if ref $friend ne "Mojo::Webqq::Friend";
+    for(my $i=0;@{$self->friend};$i++){
+        if($friend->id eq $self->friend->[$i]->id){
+            splice @{$self->friend},$i,1;
+            return 1; 
+        }
+    }
+    return 0;
+}
 sub add_friend {
     my $self = shift;
     my $friend = shift;
@@ -177,9 +207,11 @@ sub update_friend_ext {
             next if not exists $ext->{$id};
             $_->{qq} = $ext->{$id}{qq};
         }
+        $self->emit("model_update"=>"friend_ext",1);
     }
     else{
         $self->warn("更新好友扩展信息失败");
+        $self->emit("model_update"=>"friend_ext",0);
     }
 }
 sub update_friend {
@@ -190,7 +222,7 @@ sub update_friend {
         $self->info("更新好友 [ " . $friend->nick .  " ] 信息...\n");
         my $friend_info = $self->_get_friend_info($friend->id);  
         if(defined $friend_info){$friend->update($friend_info);}
-        else{$self->warn("更新好友 [ " . $friend->nick .  " ] 信息失败...\n");$self->emit("model_update_fail");}
+        else{$self->warn("更新好友 [ " . $friend->nick .  " ] 信息失败...\n");}
         return $self;
     }
     my @friends;
@@ -199,17 +231,27 @@ sub update_friend {
     if(defined $friends_info){
         push @friends,$self->new_friend($_) for @{$friends_info};
         if(ref $self->friend eq "ARRAY" and @{$self->friend}  == 0){
-            #@{$self->friend} = @{$friends_info}; 
             $self->friend(\@friends);
         }
         else{
-            my($new_friends,$lost_friends) = $self->array_diff($self->friend,\@friends,sub{$_[0]->id});
-            $self->emit(new_friend=>$_) for @{$new_friends};
-            $self->emit(lose_friend=>$_) for @{$lost_friends};
-            $self->friend(\@friends);
+            my($new_friends,$lost_friends,$sames) = $self->array_diff($self->friend,\@friends,sub{$_[0]->id});
+            for(@{$new_friends}){
+                $self->add_friend($_);
+                $self->emit(new_friend=>$_);
+            }
+            for(@{$lost_friends}){
+                $self->remove_friend($_);
+                $self->emit(lose_friend=>$_);
+            }
+            for(@{$sames}){
+                my($old,$new) = @$_;
+                $old->update($new);
+            }
         }
+        $self->update_friend_ext();
+        $self->emit("model_update","friend",1);
     }
-    else{$self->warn("更新好友信息失败\n");$self->emit("model_update_fail");}
+    else{$self->warn("更新好友信息失败\n");$self->emit("model_update","friend",0);}
 }
 sub search_friend {
     my $self = shift;
@@ -246,10 +288,22 @@ sub add_group{
     return $self;
 }
 
+sub remove_group{
+    my $self = shift;
+    my $group = shift;
+    $self->die("不支持的数据类型") if ref $group ne "Mojo::Webqq::Group";
+    for(my $i=0;@{$self->group};$i++){
+        if($group->gid eq $self->group->[$i]->gid){
+            splice @{$self->group},$i,1;
+            return 1;
+        }
+    }
+    return 0;
+}
 sub update_group_ext {
     my $self = shift;
     return if not $self->is_support_model_ext;
-    $self->info("更新群扩展信息...\n");
+    $self->info("更新群扩展信息...");
     my $group_list_ext = $self->_get_group_list_info_ext();
     if(defined $group_list_ext and ref $group_list_ext eq "ARRAY"){
         for my $g(@$group_list_ext){
@@ -259,7 +313,7 @@ sub update_group_ext {
                     $g->{$_} = $group_info_ext->{$_}; 
                 }  
             }
-            else{$self->warn("更新[ $g->{gname} ]扩展信息失败")}
+            else{$self->warn("更新[ $g->{gname} ]扩展信息失败");}
         }
         my(undef,$gext)= $self->array_unique($group_list_ext,sub{$_[0]->{gname} . $_[0]->{member_count}});
         my $unique_group = $self->array_unique($self->group,sub{$_[0]->gname . @{$_[0]->member}});
@@ -285,8 +339,9 @@ sub update_group_ext {
                 $_->{gnumber} = $g->gnumber;
             }
         }
+        $self->emit("model_update","group_ext",1);
     }
-    else{$self->warn("更新群扩展信息失败\n");}
+    else{$self->warn("更新群扩展信息失败\n");$self->emit("model_update","group_ext",0);}
 
 }
 sub update_group {
@@ -303,7 +358,6 @@ sub update_group {
         }
         else{
             $self->warn("更新群 [ " . $group->gname .  " ] 信息失败...");
-            #$self->emit("model_update_fail");
         }
         return $self;
     }
@@ -312,7 +366,7 @@ sub update_group {
     my $group_list = $self->_get_group_list_info(); 
     unless(defined $group_list){
         $self->warn("更新群列表信息失败\n");
-        $self->emit("model_update_fail");
+        $self->emit("model_update","group",0);
         return $self;
     }
     for my $g (@{$group_list}){
@@ -321,7 +375,6 @@ sub update_group {
         $group_info = $self->_get_group_info($g->{gcode});
         unless(defined $group_info){
             $self->warn("更新[ " . $g->{gname} . " ]信息失败\n");
-            #$self->emit("model_update_fail");
             $group_info = $g;
         }
         if(ref $group_info->{member} ne 'ARRAY'){
@@ -334,26 +387,38 @@ sub update_group {
     }
     else{
         my($new_groups,$lost_groups,$sames) = $self->array_diff($self->group,\@groups,sub{$_[0]->gid});  
-        $self->emit(new_group=>$_) for @{$new_groups};
-        $self->emit(lose_group=>$_) for @{$lost_groups};
-        for (
-            grep { ref($_->[0]->member) eq "ARRAY"
-                and ref($_->[1]->member) eq "ARRAY"
-                and @{$_->[0]->member}!=0 
-            } @{$sames}
-        ){
-            if(@{$_->[1]->member}!=0){
-                my($old_group,$new_group) = ($_->[0],$_->[1]);
-                my($new_members,$lose_members) = $self->array_diff($old_group->member,$new_group->member,sub{$_[0]->id});
-                $self->emit(new_group_member=>$_) for @{$new_members};
-                $self->emit(lose_group_member=>$_) for @{$lose_members};
-            }
-            else{
-                $_->[1]->member($_->[0]->member);
-            }
+        for(@{$new_groups}){
+            $self->add_group($_);
+            $self->emit(new_group=>$_) ;
         }
-        $self->group(\@groups);
+        for(@{$lost_groups}){
+            $self->remove_group($_);
+            $self->emit(lose_group=>$_) ;
+        }
+        for(@{$sames}){
+            my($old_group,$new_group) = ($_->[0],$_->[1]);
+            $old_group->update($new_group); 
+        }
+        #for (
+        #    grep { ref($_->[0]->member) eq "ARRAY"
+        #        and ref($_->[1]->member) eq "ARRAY"
+        #        and @{$_->[0]->member}!=0 
+        #    } @{$sames}
+        #){
+        #    if(@{$_->[1]->member}!=0){
+        #        my($old_group,$new_group) = ($_->[0],$_->[1]);
+        #        my($new_members,$lose_members)=$self->array_diff($old_group->member,$new_group->member,sub{$_[0]->id});
+        #        $self->emit(new_group_member=>$_) for @{$new_members};
+        #        $self->emit(lose_group_member=>$_) for @{$lose_members};
+        #    }
+        #    else{
+        #        $_->[1]->member($_->[0]->member);
+        #    }
+        #}
+        #$self->group(\@groups);
     }
+    $self->update_group_ext();
+    $self->emit("model_update","group",1);
 }
 
 sub search_group {
@@ -370,18 +435,6 @@ sub search_group {
     }
 }
 
-sub remove_group_member{
-    my $self = shift;
-    my $member = shift;
-    $self->die("不支持的数据类型") if not $member->is_group_member;
-    for(my $i=0;$i<@{$member->group->member};$i++){
-        if($member->group->member->[$i]->id eq $member->id){
-            splice @{$member->group->member},$i,1;
-            return 1;
-        }
-    }
-    return;
-}
 sub search_group_member {
     my $self = shift;
     my %p = @_;
@@ -417,7 +470,6 @@ sub update_discuss {
         }
         else{
             $self->warn("更新讨论组 [ " . $discuss->dname .  " ] 信息失败...");
-            #$self->emit("model_update_fail");
         }
         return $self;
     }
@@ -426,7 +478,7 @@ sub update_discuss {
     my $discuss_list = $self->_get_discuss_list_info(); 
     unless(defined $discuss_list){
         $self->warn("更新讨论组列表信息失败\n");
-        $self->emit("model_update_fail");
+        $self->emit("model_update","discuss",0);
         return $self;
     }
     for my $d (@{$discuss_list}){
@@ -447,22 +499,33 @@ sub update_discuss {
     }
     else{
         my($new_discusss,$lost_discusss,$sames) = $self->array_diff($self->discuss,\@discusss,sub{$_[0]->did});  
-        $self->emit(new_discuss=>$_) for @{$new_discusss};
-        $self->emit(lose_discuss=>$_) for @{$lost_discusss};
-        for (
-            grep { ref($_->[0]->member) eq "ARRAY"
-                and ref($_->[1]->member) eq "ARRAY"
-                and @{$_->[0]->member}!=0 
-                and @{$_->[1]->member}!=0
-            } @{$sames}
-        ){
-            my($old_discuss,$new_discuss) = ($_->[0],$_->[1]);
-            my($new_members,$lose_members) = $self->array_diff($old_discuss->member,$new_discuss->member,sub{$_[0]->id});
-            $self->emit(new_discuss_member=>$_) for @{$new_members};
-            $self->emit(lose_discuss_member=>$_) for @{$lose_members};
+        for(@{$new_discusss}){
+            $self->add_discuss_member($_);
+            $self->emit(new_discuss=>$_);   
         }
-        $self->discuss(\@discusss);
+        for(@{$lost_discusss}){
+            $self->remove_discuss_member($_);
+            $self->emit(lose_discuss=>$_);
+        }
+        for(@{$sames}){
+            my($old_discuss,$new_discuss) = ($_->[0],$_->[1]);
+            $old_discuss->update($new_discuss);
+        }
+        #for (
+        #    grep { ref($_->[0]->member) eq "ARRAY"
+        #        and ref($_->[1]->member) eq "ARRAY"
+        #        and @{$_->[0]->member}!=0 
+        #        and @{$_->[1]->member}!=0
+        #    } @{$sames}
+        #){
+        #    my($old_discuss,$new_discuss) = ($_->[0],$_->[1]);
+        #    my($new_members,$lose_members)= $self->array_diff($old_discuss->member,$new_discuss->member,sub{$_[0]->id});
+        #    $self->emit(new_discuss_member=>$_) for @{$new_members};
+        #    $self->emit(lose_discuss_member=>$_) for @{$lose_members};
+        #}
+        #$self->discuss(\@discusss);
     }
+    $self->emit("model_update","discuss",1);
 }
 
 sub search_discuss {
@@ -517,10 +580,11 @@ sub update_recent {
             #elsif($_->{type} eq "group"){}
             #elsif($_->{type} eq "discuss"){}
         }
+        $self->emit("model_update","recent",1);
     }
     else{
         $self->warn("更新最近联系人信息失败\n");
-        $self->emit("model_update_fail");
+        $self->emit("model_update","recent",0);
     }
 }
 
